import { Module } from "vuex";
import { FETCH_ACCOUNT, FETCH_BUDGET, FETCH_MONTH_BUDGET, SET_CURRENT_ACCOUNT } from "../action-types";
import { LOGOUT, TITLE } from "../mutation-types";

export interface BudgetState {
    Accounts: Map<string, Account>,
    CurrentAccountID?: string,
    Categories: Map<string, Category>,
    Months: Map<number, Map<number, Map<string, Category>>>,
    Transactions: [],
    Assignments: []
}

export interface Account {
    ID: string
    OnBudget: boolean
}

export interface Category {
    Group: string
    Name: string
    AvailableLastMonth: number
    Assigned: number
    Activity: number
    Available: number
}

export const budgetStore : Module<BudgetState, any> = {
    state: {
        Accounts: new Map<string, Account>(),
        CurrentAccountID: undefined,
        Months: new Map<number, Map<number, Map<string, Category>>>(),
        Categories: new Map<string, Category>(),
        Transactions: [],
        Assignments: []
    },

    mutations: {
        initializeStore(state) {
            const store = localStorage.getItem("store");
            if (!store)
                return;

            const restoredState = JSON.parse(store);
            if (!restoredState)
                return;

            state.CurrentAccountID = restoredState.CurrentAccountID;

            for (const account of restoredState.Accounts || []) {
                state.Accounts.set(account[0], account[1]);
            }
        },
        [LOGOUT](state) {
            state.Accounts.clear();
            state.Categories.clear();
            state.Transactions = [];
            state.Assignments = [];
        },
        addAccount(state, account) {
            state.Accounts.set(account.ID, account);
        },
        addCategory(state, category) {
            state.Categories.set(category.ID, category);
        },
        addCategoriesForMonth(state, {year, month, categories}) {
            const yearMap = state.Months.get(year) || new Map<number, Map<string, Category>>();
            state.Months.set(year, yearMap);

            const monthMap = yearMap.get(month) || new Map<string, Category>();
            yearMap.set(month, monthMap);

            for (const category of categories){
                monthMap.set(category.ID, category);
            }
        },
        setCurrentAccountID(state, accountid) {
            state.CurrentAccountID = accountid;
        },
        setTransactions(state, transactions) {
            state.Transactions = transactions;
        }
    },

    getters: {
        Accounts(state) {
            return state.Accounts.values();
        },
        Categories: (state) => (year : number, month : number) => {
            const yearMap = state.Months.get(year);
            return yearMap?.get(month)?.values();
        },
        CurrentAccount(state) : Account | undefined {
            if (state.CurrentAccountID == null)
                return undefined;
            return state.Accounts.get(state.CurrentAccountID);
        },
        OnBudgetAccounts(state) {
            return Array.from(state.Accounts.values()).filter(x => x.OnBudget);
        },
        OnBudgetAccountsBalance(state, getters){
            return getters.OnBudgetAccounts.reduce((prev, curr) => prev + Number(curr.Balance), 0);
        },
        OffBudgetAccounts(state) {
            return Array.from(state.Accounts.values()).filter(x => !x.OnBudget);
        },
        OffBudgetAccountsBalance(state, getters){
            return getters.OffBudgetAccounts.reduce((prev, curr) => prev + Number(curr.Balance), 0);
        },
        Transactions(state) {
            return (state.Transactions || []);
        }
    },

    actions: {
        async [SET_CURRENT_ACCOUNT]({ state, commit, dispatch, getters }, { budgetid, accountid }) {
            if (budgetid == null)
                return

            commit("setCurrentAccountID", accountid);
            if (accountid == null)
                return

            commit(TITLE, getters.CurrentAccount.Name);
            await dispatch(FETCH_ACCOUNT, accountid)
        },
        async [FETCH_ACCOUNT]({ state, commit, rootState }, accountid) {
            const result = await fetch("/api/v1/account/" + accountid + "/transactions", {
                headers: {
                    'Authorization': 'Bearer ' + rootState.Session.Token
                }
            });
            const response = await result.json();
            commit("setTransactions", response.Transactions);
        },
        async [FETCH_BUDGET]({ state, commit, dispatch, rootState }, budgetid) {
            const result = await dispatch("GET", { path: "/budget/" + budgetid });
            const response = await result.json();
            for (const account of response.Accounts || []) {
                commit("addAccount", account);
            }
            for (const category of response.Categories || []) {
                commit("addCategory", category);
            }
        },
        async [FETCH_MONTH_BUDGET]({state, commit, dispatch, rootState }, {budgetid, month, year}) {
            const result = await dispatch("GET", { path: "/budget/" + budgetid + "/" + year + "/" + month});
            const response = await result.json();
            commit("addCategoriesForMonth", {year, month, categories: response.Categories})
        }
    }
}